@@ -175,7 +175,7 @@ void Utility::cancelWindowMoveResize(quint32 WId)
     sendMoveResizeMessage(WId, _NET_WM_MOVERESIZE_CANCEL);
 }
 
-void Utility::updateMousePointForWindowMove(quint32 WId)
+void Utility::updateMousePointForWindowMove(quint32 WId, bool finished/* = false*/)
 {
     xcb_client_message_event_t xev;
     const QPoint &globalPos = qApp->primaryScreen()->handle()->cursor()->pos();
@@ -186,7 +186,8 @@ void Utility::updateMousePointForWindowMove(quint32 WId)
     xev.format = 32;
     xev.data.data32[0] = globalPos.x();
     xev.data.data32[1] = globalPos.y();
-    xev.data.data32[2] = 0;
+    // fix touch screen moving dtk window : 0 update position, 1 moving finished and clear moving state
+    xev.data.data32[2] = finished ? 1 : 0;
     xev.data.data32[3] = 0;
     xev.data.data32[4] = 0;
 
@@ -323,6 +324,8 @@ void Utility::sendMoveResizeMessage(quint32 WId, uint32_t action, QPoint globalP
         globalPos = qApp->primaryScreen()->handle()->cursor()->pos();
     }
 
+    qDebug() << Q_FUNC_INFO << globalPos;
+
     xcb_client_message_event_t xev;
 
     xev.response_type = XCB_CLIENT_MESSAGE;
@@ -618,6 +621,23 @@ bool Utility::blurWindowBackgroundByImage(const quint32 WId, const QRect &blurRe
     return true;
 }
 
+bool Utility::updateBackgroundWallpaper(const quint32 WId, const QRect &area, const quint32 bMode)
+{
+    xcb_atom_t atom = DXcbWMSupport::instance()->_deepin_wallpaper;
+
+    if (atom == XCB_NONE)
+        return false;
+
+    QVector<quint32> rects;
+    quint32 window_mode = (bMode & 0xffff0000) >> 16; //High 16 bits
+    quint32 wallpaper_mode = bMode & 0x0000ffff;      //low 16 bits
+    rects << area.x() << area.y() << area.width() << area.height() << window_mode << wallpaper_mode;
+
+    setWindowProperty(WId, atom, XCB_ATOM_CARDINAL, rects.constData(), rects.size(), sizeof(quint32) * 8);
+
+    return true;
+}
+
 void Utility::clearWindowBlur(const quint32 WId)
 {
     clearWindowProperty(WId, DXcbWMSupport::instance()->_net_wm_deepin_blur_region_rounded_atom);
@@ -625,6 +645,11 @@ void Utility::clearWindowBlur(const quint32 WId)
     clearWindowProperty(WId, DXcbWMSupport::instance()->_kde_net_wm_blur_rehind_region_atom);
 }
 
+void Utility::clearWindowBackground(const quint32 WId)
+{
+    clearWindowProperty(WId, DXcbWMSupport::instance()->_deepin_wallpaper);
+}
+
 qint32 Utility::getWorkspaceForWindow(quint32 WId)
 {
     xcb_get_property_cookie_t cookie = xcb_get_property(DPlatformIntegration::xcbConnection()->xcb_connection(), false, WId,
